(* Program to convert drbar masses of stops and gluinos to pole masses. 
 *
 * Uses TSIL to evaluate self-energies
 *
 * relevant papers:
 *
 *   [EFFPOT] S.P. Martin, " Two loop effective potential for the 
 *     minimal supersymmetric standard model", hep-ph/0206136
 *   [INTEGRALS] S.P. Martin, "Evaluation of two-loop self-energy 
 *     basis integrals using differential equations",
 *     hep-ph/0307101
 *   [TSIL] S.P. Martin, "TSIL: Aa program for the calculation of
 *     two-loop self-energy integrals", hep-ph/0501132
 *   [FERMION] S.P. Martin, "Fermion self-energies and pole masses
 *     at two-loop order in a general renormalisable theory with
 *     massless gauge bosons", hep-ph/0509115
 *   [SCALAR] S.P. Martin, "Two-loop scalar self-energies and pole
 *     masses in a general renormalisable theory with massless 
 *     gauge bosons", hep-ph/0502168
 *   [SCALAR'] S.P. Martin, " Two loop scalar self energies in a 
 *     general renormalizable theory at leading order in gauge 
 *     couplings", hep-ph/0312092
 *
 *********************************************************************
 *
 * Gluinos are treated at one loop, stops at two loops, since
 * that is what you need for the three loop higgs mass.
 *
 * In the following, we describe the notation. Capital M denote
 * OS masses, small m denote DRbar masses.
 * a := Alpha_s/(4 Pi)
 * 
 * Mgl = zgl mgl
 *     = (1 + a zgl1[mgl,mst1]  ) mgl
 *
 * Msti = zsti msti
 *      = (1 + a zsti1[mgl,mst{i}] + a^2 zsti2[mgl,mst{i},msq] ) msti
 *
 * in order to express the running stop mass through the on-shell
 * masses consistently up to two loops, the drbar masses in zst1
 * have to be replaced by the 1 loop os masses.
 *
 * What we want are the conversions from drbar to os, that is, z^-1, 
 *
 * zgl^-1 = 1 - a zgl1[mgl, mst]
 * zsti^-1 = 1 - a zsti1[Mgl, Mst{i}] 
 *        + a^2 ( Zsti1[mgl, mst{i}]^2 - Zsti2[mgl, mst{i}] )
 *
 *)

(* {{{ set up all relevant constants *)

(* {{{ paths to the TSIL programs that evaluate the integrals *)
$EvalBCmd = "~/q2eexpcalc3/calc3/problems/c_mh/math/tsil/evalB";
$EvalBpCmd = "~/q2eexpcalc3/calc3/problems/c_mh/math/tsil/evalBp";
$EvalI2Cmd = "~/q2eexpcalc3/calc3/problems/c_mh/math/tsil/evalI2";
$EvalI2pCmd = "~/q2eexpcalc3/calc3/problems/c_mh/math/tsil/evalI2p";
$EvalSCmd = "~/q2eexpcalc3/calc3/problems/c_mh/math/tsil/evalS";
$EvalTCmd = "~/q2eexpcalc3/calc3/problems/c_mh/math/tsil/evalT";
$EvalUCmd = "~/q2eexpcalc3/calc3/problems/c_mh/math/tsil/evalU";
$EvalVCmd = "~/q2eexpcalc3/calc3/problems/c_mh/math/tsil/evalV";
$EvalMCmd = "~/q2eexpcalc3/calc3/problems/c_mh/math/tsil/evalM";
(* }}} *)
(* {{{ lagrangian parameters *)

(* === squark mixing === *)
(* entries of mixing matrices, see Eq. (2.41) of [EFFPOT] 
 * we assume a real matrix, and thus use a mixing angle 
 *)
L[q_, 1] := Cos[theta[q]];
L[q_, 2] := - Sin[theta[q]];
R[q_, 1] := Sin[theta[q]];
R[q_, 2] := Cos[theta[q]];
(* mixing angles *)
theta[U]=0;
theta[D]=0;
theta[S]=0;
theta[C]=0;
theta[T]=0;
theta[B]=0;
(* shorthand notation defined in [SCALAR], eqs. (4.5, 4.6) *)
PP[q_, i_, j_] := L[q, i] L[q, j] - R[q, i] R[q, j];
NN[q_, i_, j_] := L[q, i] R[q, j] + R[q, i] L[q, j];
(* === quark and squark squared masses === *)
Do[ quark[q] = 1.*10^(-3);
    squark[q,1]=msq^2;
    squark[q,2]=msq^2;
    ,
    {q, {U,D,S,C,B} } ];

quark[T]=1.*10^(-3);
squark[T,1]=mst1^2;
squark[T,2]=mst2^2;
(* }}} *)

su3 = {tr -> 1/2,
       ca -> 3,
       cf -> 4/3
       };

precision=10^(-3);

(* }}} *)


(* {{{ functions *)

(* function to convert a real number to a string that can be passed 
 * to a c program *)
realToString[x_]:=ToString[NumberForm[x, NumberFormat->(SequenceForm[#1, "E", #3] &)]]
(* Kronecker delta *)
delta[i_?IntegerQ, j_?IntegerQ] := If[ i == j, 1, 0 ];
(* this function sums only terms that depend on the summation index,
 * and adds the 'constant' term (once) *)
SumDep[expr_, index_List, furtherindex__] := SumDep[SumDep[
    expr, index], furtherindex];
SumDep[expr_, index_List] := If[
    (* catch the case where FreeQ would fail *)
    AtomQ[expr],
    If[expr === index[[1]],
       Sum[expr, index],
       expr],
    (* the 'general' case *)
    Plus[
        Select[Expand[expr], FreeQ[#, index[[1]]] & ],
        Summ[
            Select[Expand[expr], !FreeQ[#, index[[1]]] & ],
            index]
    ]/.Summ->Sum];
trim[expr_]:=Chop[Expand[expr],precision];
print[expr__]:=Print[expr]; (* set to null to diable debugging *)

(* === functions in S.P.Martin's notation === 
 * the parameters s and qq, which are suppressed in the papers
 * [TSIL, FERMION, SCALAR], are restored and appear as the last
 * two arguments in the following functions
 *)

(* === integral functions === *)
ln[x_?NumberQ, y_?NumberQ] := Log[x/y];
TsilA[x_, qq_] := x(ln[x,qq]-1);

TsilB[x_?NumberQ, y_?NumberQ, s_?NumberQ, qq_?NumberQ] := TsilB[x, y, s, qq] =
    RunThrough[($EvalBCmd <> " "
                <>realToString[x] <> " "
                <>realToString[y] <> " "
                <>realToString[s] <> " "
                <>realToString[qq]), Null ];

TsilBp[x_?NumberQ, y_?NumberQ, s_?NumberQ, qq_?NumberQ] := TsilBp[x, y, s, qq] =
    RunThrough[($EvalBpCmd <> " "
                <>realToString[x] <> " "
                <>realToString[y] <> " "
                <>realToString[s] <> " "
                <>realToString[qq]), Null ];

TsilI2[x_?NumberQ, y_?NumberQ, z_?NumberQ, qq_?NumberQ] := TsilI2[x, y, s, qq] =
    RunThrough[($EvalI2Cmd <> " "
                <>realToString[x] <> " "
                <>realToString[y] <> " "
                <>realToString[z] <> " "
                <>realToString[qq]), Null ];
               
TsilI2p[x_?NumberQ, y_?NumberQ, z_?NumberQ, qq_?NumberQ] := TsilI2p[x, y, s, qq] =
    RunThrough[($EvalI2pCmd <> " "
                <>realToString[x] <> " "
                <>realToString[y] <> " "
                <>realToString[z] <> " "
                <>realToString[qq]), Null ];
               
TsilS[x_?NumberQ, y_?NumberQ, z_?NumberQ, s_?NumberQ, qq_?NumberQ] := 
    TsilS[x, y, z, s, qq] =
    RunThrough[($EvalSCmd <> " "
                <>realToString[x] <> " "
                <>realToString[y] <> " "
                <>realToString[z] <> " "
                <>realToString[s] <> " "
                <>realToString[qq]), Null ];

TsilT[x_?NumberQ, y_?NumberQ, z_?NumberQ, s_?NumberQ, qq_?NumberQ] := 
    TsilT[x, y, z, s, qq] =
    RunThrough[($EvalTCmd <> " "
                <>realToString[x] <> " "
                <>realToString[y] <> " "
                <>realToString[z] <> " "
                <>realToString[s] <> " "
                <>realToString[qq]), Null ];
               
TsilU[x_?NumberQ, y_?NumberQ, z_?NumberQ, u_?NumberQ, s_?NumberQ, qq_?NumberQ] := 
    TsilU[x, y, z, u, s, qq] =
    RunThrough[($EvalUCmd <> " "
                <>realToString[x] <> " "
                <>realToString[y] <> " "
                <>realToString[z] <> " "
                <>realToString[u] <> " "
                <>realToString[s] <> " "
                <>realToString[qq]), Null ];
               
TsilV[x_?NumberQ, y_?NumberQ, z_?NumberQ, u_?NumberQ, s_?NumberQ, qq_?NumberQ] := 
    TsilV[x, y, z, u, s, qq] =
    RunThrough[($EvalVCmd <> " "
                <>realToString[x] <> " "
                <>realToString[y] <> " "
                <>realToString[z] <> " "
                <>realToString[u] <> " "
                <>realToString[s] <> " "
                <>realToString[qq]), Null ];
               
TsilM[x_?NumberQ, 
      y_?NumberQ, 
      z_?NumberQ, 
      u_?NumberQ, 
      v_?NumberQ, 
      s_?NumberQ, 
      qq_?NumberQ] := 
    TsilV[x, y, z, u, v, s, qq] =
    RunThrough[($EvalMCmd <> " "
                <>realToString[x] <> " "
                <>realToString[y] <> " "
                <>realToString[z] <> " "
                <>realToString[u] <> " "
                <>realToString[v] <> " "
                <>realToString[s] <> " "
                <>realToString[qq]), Null ];
               
(* === replacement rules to express everything through TSIL functions === *)
toTSIL = 
{
(* Eqs. (3.3, 3.4) of [FERMION] *)
     BFS[x_, y_, s_, qq_] -> ((y - x - s) TsilB[x, y, s, qq] - TsilA[x, qq] + TsilA[y, qq])/2,
     BFbarS[x_, y_, s_, qq_] -> - TsilB[x, y, s, qq],
(* Eqs. (3.4 - 3.6) of [SCALAR] *)
     BSS[x_, y_, s_, qq_] -> - TsilB[x, y, s, qq],
     BFbarFbar[x_, y_, s_, qq_] -> 2 TsilB[x, y, s, qq],
     BFF[x_, y_, s_, qq_] -> (x + y - s) TsilB[x, y, s, qq] - TsilA[x, qq] - TsilA[y, qq],
     BSV[x_, 0, s_, qq_] -> ((3 - xi) (x + s) TsilB[0, x, s, qq]
                               + (3 - 2 xi) TsilA[x, qq]
                             + 2 (xi - 1) s ),

(* Eqs. (4.3, 4.11) of [SCALAR'] *)
     XSSS[x_, y_, z_, qq_] -> If[ 
         x == y,
         TsilA[z, qq] ln[x, qq],
         TsilA[z, qq] (TsilA[x, qq] - TsilA[y, qq]) / (x-y)],
     
(* Eqs. (4.16, 4.17) of [SCALAR'] *)
     WSSFF[x_, y_, z_, u_, qq_] -> If[
         x == y,
         ( (z + u - x) TsilI2p[x, z, u, qq] - TsilI2[x, z, u, qq]
          -( TsilA[u, qq] + TsilA[z, qq] ) ln[x, qq]
          ),
         ( ( (z + u - x) TsilI2[x, z, u, qq] 
            - TsilA[x, qq]( TsilA[z, qq] + TsilA[u, qq] ) )/(x - y) 
          +( (z + u - y) TsilI2[y, z, u, qq] 
            - TsilA[y, qq]( TsilA[z, qq] + TsilA[u, qq] ) )/(y - x) 
          )],
(* Eqs. (4.2, 4.10, 4.15) of [SCALAR'] *)
     WSSSS[x_, y_, z_, u_, qq_] -> If[
         x == y,
         ( - TsilI2p[x, z, u, qq] ),
         ( (TsilI2[x, z, u, qq] - TsilI2[y, z, u, qq])/(y - x) )],
     WSSFbarFbar[x_, y_, z_, u_, qq_] -> - 2 WSSSS[x, y, z, u, qq],
(* Vbar from Appendix A, [SCALAR'], Eqs. (A.11 - A.13) *)
     Vbar[x_, 0, z_, u_, s_, qq_] -> Which[
         ( (z == 0) && (u == 0) ),
         ( ( -(s+x) TsilTbar[0,0,x,s,qq] + 2 TsilS[0,0,x,s,qq]
            -(s+x) TsilB[0,x,s,qq] -3 TsilA[x,qq] 
            +x-s/4 )/(s-x)^2 ),
         ( u == z ),
         (
          1/(3 z (s-x)^2) 
          ( (4 z-3 x-s) z TsilT[z,z,x,s,qq]
           +(s-x) x TsilT[x,z,z,s,qq] 
           + (4 z-x+s) TsilS[x,z,z,s,qq]
           -2 z TsilI2[0,z,z,qq]
           + 2 TsilA[z,qq] ( x-2 z-TsilA[x,qq])
           + TsilA[x,qq] ( x/2-4 z-s/2 )
           + 4 z^2-z x - x^2 
           + 13 s x/8 + s z/2 - 5 s^2/8
           )
          - TsilB[0,x,s,qq] ( ( TsilA[z,qq]/z+1) (s+x)/(s-x)^2 + 1/6 z )
          ),
         True,
         (
          -x TsilU[x,0,z,u,s,qq]/(s-x)^2 
          +( (z+u)/((z-u)^2(s-x)) - 1/((s-x)^2) ) x TsilT[x,z,u,s,qq]
          +( (u(z-u-x+s))/((z-u)^3 (s-x)) -(2 x)/((z-u)(s-x)^2)
            ) z TsilT[z,x,u,s,qq]
          
          +( (z(z-u+x-s))/((z-u)^3 (s-x)) +(2 x)/((z-u)(s-x)^2) 
            ) u TsilT[u,x,z,s,qq]
          + 
          (
           (z+u) ( TsilS[x,z,u,s,qq] -TsilA[x,qq]/2 +x+z+u-5 s/8 ) + TsilA[z,qq] TsilA[u,qq] 
           )/((z-u)^2(s-x))
          + TsilB[0,x,s,qq] (((2 x+s) ( TsilA[u,qq]-TsilA[z,qq] ))/((s-x)^2 (z-u)) 
                             + (u TsilA[z,qq] - z TsilA[u,qq])/((z-u)^3)
                             - ((z+u))/(2(z-u)^2) )
          + (TsilA[x,qq] ( TsilA[u,qq] - TsilA[z,qq] ))/((z-u)(s-x)^2)
          + 
          (
           (x z+x u-2 s u) TsilA[z,qq] 
           + (x z+x u-2 s z) TsilA[u,qq] 
           )/((z-u)^2(s-x)^2)
          )],
(* Eqs. (4.44, 4.43, 4.38) of [SCALAR'] *)
     VFFFFS[x_, y_, z_, u_, v_, s_, qq_] -> Which[
         ( (y == 0) && (z == 0) ),
         ((
           (u-v)(x-s) Vbar[x,0,u,v,s,qq]
           + (s - x - u +v) TsilU[x,0,u,v,s,qq]
           +TsilS[x,u,v,s,qq]
           +TsilI2[0,u,v,qq] )/2
          + (TsilA[u,qq] - TsilA[v,qq])(s TsilBp[x,0,s,qq] + TsilA[x,qq]/x)
          ),
         ( z == y ),
         (
          (
           (y+u-v) ((x+y-s) TsilV[x,y,u,v,s,qq] + TsilI2p[y,u,v,qq])
           + (s - x - 2 y - u +v) TsilU[x,y,u,v,s,qq]
           +TsilS[x,u,v,s,qq]
           +TsilI2[y,u,v,qq]
           + (TsilA[u,qq] - TsilA[v,qq])((s-x-y)TsilBp[y,x,s,qq] -TsilB[x,y,s,qq]
                                         + ln[y,qq])
           )/2.
          ),
         True,
         (
          (
           (y+u - v) ((s-x -y) TsilU[x,y,u,v,s,qq]+ TsilI2[y,u,v,qq]) + y TsilS[x,u,v,s,qq]
           + (TsilA[u,qq]-TsilA[v,qq])( (s-x-y) TsilB[x,y,s,qq] + TsilA[y,qq] )
           )/(2(y-z))
          +
          (
           (x+u - v) ((s-y-x) TsilU[y,x,u,v,s,qq]+ TsilI2[x,u,v,qq]) + x TsilS[y,u,v,s,qq]
           + (TsilA[u,qq]-TsilA[v,qq])( (s-y-x) TsilB[y,x,s,qq] + TsilA[x,qq] )
           )/(2(x-z))
          )
         ],
(* Eqs. (4.42, 4.37) [SCALAR'] *)
     VFFFbarFbarS[x_, y_, z_, u_, v_, s_, qq_] -> Which[
         ( z == y ),
         (
          (x+y-s) TsilV[x,y,u,v,s,qq] - TsilU[x,y,u,v,s,qq] + 
          TsilI2p[y,u,v,qq]
          ),
         True,
         (
          ( (s-x-y) TsilU[x,y,u,v,s,qq] + TsilI2[y,u,v,qq] )/(y-z) 
          +
          ( (s-x-z) TsilU[x,z,u,v,s,qq] + TsilI2[z,u,v,qq] )/(z-y) 
          )
         ],
(* Eqs. (4.40, 4.35) [SCALAR'] *)
     VFbarFFFbarS[x_, y_, z_, u_, v_, s_, qq_] -> Which[
         ( z == y ),
         (
          2 y TsilV[x,y,u,v,s,qq] - 2 TsilU[x,y,u,v,s,qq]
          ),
         True,
         (
          2 (z TsilU[x,z,u,v,s,qq] - y TsilU[x,y,u,v,s,qq])/(y-z)
          )
         ],
(* Eq. (4.33) [SCALAR'] *)
     VFbarFbarFbarFbarS[x_, y_, z_, u_, v_, s_, qq_] -> 
         -2 VSSSSS[x,y,z,u,v,s,qq],
(* Eqs. (4.13, 4.8) [SCALAR'] *)
     VSSSSS[x_, y_, z_, u_, v_, s_, qq_] -> Which[
         ( z == y ),
         ( -TsilV[x,y,u,v,s,qq] ),
         True,
         ( (TsilU[x,y,u,v,s,qq] -TsilU[x,z,u,v,s,qq])/(y - z) )
         ],
(* Eq. (4.23) [SCALAR'] *)
     MFFFFS[x_, y_, z_, u_, v_, s_, qq_] ->
         (
          (x u +y z- v s) TsilM[x,y,z,u,v,s,qq] -x TsilU[z,x,y,v,s,qq] -
          z TsilU[x,z,u,v,s,qq] -u TsilU[y,u,z,v,s,qq] -y TsilU[u,y,x,v,s,qq] + 
          TsilS[x,u,v,s,qq] + TsilS[y,z,v,s,qq] + s TsilB[x,z,s,qq] TsilB[y,u,s,qq]
          ),
(* Eq. (4.22) [SCALAR'] *)
     MFFFbarFbarS[x_, y_, z_, u_, v_, s_, qq_] ->
         (
          (x+y-v) TsilM[x,y,z,u,v,s,qq] - TsilU[x,z,u,v,s,qq] - 
          TsilU[y,u,z,v,s,qq] + TsilB[x,z,s,qq] TsilB[y,u,s,qq]  
          ),
(* Eq. (4.20) [SCALAR'] *)
     MFFbarFbarFS[x_, y_, z_, u_, v_, s_, qq_] ->
         (
          (y+z-v-s) TsilM[x,y,z,u,v,s,qq] - 
          TsilU[x,z,u,v,s,qq] - TsilU[u,y,x,v,s,qq] + 
          TsilB[x,z,s,qq] TsilB[y,u,s,qq]
          ),
(* Eq. (4.21) [SCALAR'] *)
     MFFbarFFbarS[x_, y_, z_, u_, v_, s_, qq_] ->
         (
          (x+z-s) TsilM[x,y,z,u,v,s,qq] - TsilU[y,u,z,v,s,qq] - TsilU[u,y,x,v,s,qq]
          ),
(* Eq. (4.19) [SCALAR'] *)
     MFbarFbarFbarFbarS[x_, y_, z_, u_, v_, s_, qq_] ->
         (
          2 TsilM[x,y,z,u,v,s,qq]
          ),
(* Eqs. (4.41, 4.36) [SCALAR'] *)
     VFFbarFbarFS[x_, y_, z_, u_, v_, s_, qq_] -> Which[
         ( z == y ),
         (
          (
           y(y+u-v) ((x+y-s) TsilV[x,y,u,v,s,qq] + TsilI2p[y,u,v,qq]) +
           ((u-v)(x-s) - y^2) TsilU[x,y,u,v,s,qq] + 
           (u-v) (2 x TsilT[x,u,v,s,qq] +2TsilS[x,u,v,s,qq]-
                  TsilI2[y,u,v,qq] - TsilA[x,qq] -TsilA[u,qq]-
                  TsilA[v,qq] +x+u+v-s/4) + 
           u(x+u-v-s) TsilT[u,v,x,s,qq] - v(x+v-u-s) TsilT[v,x,u,s,qq] + 
           (TsilA[u,qq]-TsilA[v,qq])((s-x-y) y TsilBp[y,x,s,qq] + (x-s)TsilB[x,y,s,qq] +y) )/(2y^2)
          ),
         True,
         (
          (
           (y+u-v)((s-x-y) TsilU[x,y,u,v,s,qq] + TsilI2[y,u,v,qq]) + 
           (TsilA[u,qq] - TsilA[v,qq]) ((s-x-y) TsilB[x,y,s,qq] + TsilA[y,qq])
           )/( 2y(y-z) )
          +
          (
           (z+u-v)((s-x-z) TsilU[x,z,u,v,s,qq] + TsilI2[z,u,v,qq]) + 
           (TsilA[u,qq] - TsilA[v,qq]) ((s-x-z) TsilB[x,z,s,qq] + TsilA[z,qq])
           )/( 2z(z-y) )

           + ( (u-v) (2x TsilT[x,u,v,s,qq] + 2 TsilS[x,u,v,s,qq] -
                      TsilA[x,qq] -TsilA[u,qq] -TsilA[v,qq] +
                      x+u+v-s/4 ) + 
              u (x+u-v-s) TsilT[u,v,x,s,qq] - 
              v (x+v-u-s) TsilT[v,x,u,s,qq] )/(2yz)
          )
         ],
(* Eq. (4.6) [SCALAR'] *)
     SSSS[x_, y_, z_, s_, qq_] -> - TsilS[x,y,z,s,qq],
(* ---------------------------------------------------------------------- *)
(* ---query-replace-regexp needed below---------------------------------- *)
(* ---------------------------------------------------------------------- *)
(* Eq. (5.32) of [SCALAR'] *)
     GFF[x_, y_, s_, qq_] -> 
         (
          (- (x+y-s)^2 TsilM[x,x,y,y,0,s,qq] + 
           4 (x+y-s)(TsilU[x,y,y,0,s,qq] - y TsilV[x,y,y,0,s,qq]) +
           TsilS[0,x,y,s,qq] + (3x+y-s) TsilT[x,0,y,s,qq] + 
           (x+y) TsilB[x,y,s,qq]^2 + (y-x-s) TsilA[x,qq] TsilB[x,y,s,qq]/x + 
           2 (s-x-y) TsilA[x,qq] TsilBp[x,y,s,qq] - 
           5 TsilI2[0,x,x,qq] + 6 TsilA[x,qq] - 4 x -s/4
           ) + 
          (- (y+x-s)^2 TsilM[y,y,x,x,0,s,qq] + 
           4 (y+x-s)(TsilU[y,x,x,0,s,qq] -x TsilV[y,x,x,0,s,qq]) +
           TsilS[0,y,x,s,qq] + (3y+x-s) TsilT[y,0,x,s,qq] + 
           (y+x) TsilB[y,x,s,qq]^2 + (x-y-s) TsilA[y,qq] TsilB[y,x,s,qq]/y + 
           2 (s-y-x) TsilA[y,qq] TsilBp[y,x,s,qq] - 
           5 TsilI2[0,y,y,qq] + 6 TsilA[y,qq] - 4 y -s/4
           )
          ),
(* Eq. (3.33) of [SCALAR] (indeed, not the ') *)
     GS[y_,qq_] -> (y (-12 + 11 ln[y,qq] - 3 ln[y,qq]^2) ),
(* Eq. (3.43) of [SCALAR] *)
     GTildeSSFF[x_, x_, y_, z_, s_, qq_] ->
         (
          4((x-y)^2 - z^2) TsilM[0,z,y,x,z,s,qq] + 
          4 (x - y - z) TsilU[x,0,z,z,s,qq] - 4 y TsilU[z,y,z,x,s,qq] + 
          4 (x - y - z) Tbar[0,y,z,s,qq] + 
          (x - 2 z) (x - y - z) TsilT[z,0,y,s,qq]/x + 
          2 y (y + z) TsilT[y,0,z,s,qq]/x + 
          (4 (y+z)/x  -1) TsilS[0,y,z,s,qq] + 2 TsilS[x,z,z,s,qq] + 
          3 (x - y - z) TsilI2p[x,y,z,qq] + (1 -2(y+z)/x) TsilI2[x,y,z,qq] + 
          2 (x - y - z) z (3 ln[z,qq] - 5) TsilBp[z,y,s,qq] +
          (15 y + 17 z  - 7x - 
           2 (x + y + z) ln[x,qq] + (x - y - 3 z) ln[z,qq])  TsilB[y,z,s,qq] + 
          3 ln[x,qq] (y ln[y,qq] - y + z ln[z,qq] -z) - 
          y (3 + 2 (y+z)/x) ln[y,qq] + 
          z (5 ln[z,qq] -15 - 2 (y+z)/x) ln[z,qq] + 
          4 (y+z)^2/x -15 x/4  + 11 y/2 + 33 z/2
          ),
(* Eq. (3.40) of [SCALAR] *)
     GTildeSSS[x_, x_, z_, qq_] -> ( (4 - 3 ln[x,qq]) TsilA[z,qq] ),
(* Eq. (3.20-24) of [SCALAR] *)
     FTilde1[x_, qq_] ->
         (
          x (12 Pi^2 - 16 Pi^2 Log[2] - (11)/(8) + 24 Zeta[3] - 
             (39)/(2) ln[x,qq] + (15)/(2) ln[x,qq]^2
             )
          ),
     FTilde2[x_, qq_] ->
         (
          x (
             (1147)/(16) - (10 Pi^2)/(3) + 
             8 Pi^2 Log[2] - 12 Zeta[3] - 
             (409)/(12)ln[x,qq] + (19)/(4) ln[x,qq]^2
             ) 
          ),
     FTilde3[x_, y_, qq_] ->
         (
          2 (x-y^2/x) PolyLog[2,(1-x/y)] + 8 (x-y) ff[Sqrt[y/x]] + 
          y ( 18 + (Pi^2 y)/(3 x)  - 6 ln[x,qq]  - 6 ln[y,qq] ) + 
          x (-(49)/(4) - (Pi^2)/(3) + (19)/(3) ln[x,qq]  -
             2 ln[x,qq] ln[y,qq] + ln[y,qq]^2 )
          ),
     FTilde4[x_, y_, qq_] ->
         ((x + 6 y + y^2/x) PolyLog[2,(1-x/y)] + 8 (x + y) ff[Sqrt[y/x]] + 
          y (-4 - Pi^2 (1 + y/(6x)) + 7 Log[(x/y)] + 3 ln[y,qq]^2 ) + 
          x (-(75)/(8) - (Pi^2)/(6) + (25)/(6) ln[x,qq] - 
               ln[x,qq] ln[y,qq] + (1)/(2) ln[y,qq]^2 )
          ),
     ff[z_] -> 
         (
          z ( PolyLog[2, ((1-z)/(1+z))] - 
             PolyLog[2, ((z-1)/(1+z)) ] + Pi^2/4 )
          ),
(* following Eqs, (3.4, 3.11) of [SCALAR'] *)
     BFFp[x_, y_, s_, qq_] ->
         (- TsilB[x, y, s, qq] +
          (x/s + y/s - 1)(-x TsilBp[x,y,s,qq] - y TsilBp[y,x,s,qq] - 1)
          ),
(* Eq. (6.18) of [INTEGRALS] *)
     Tbar[0,x_,y_,s_,qq_] ->
         (
          (1-2 tabc[s,x,y]) PolyLog[2,tabc[x,s,y]] +
          (1-2 rabc[s,x,y]) PolyLog[2,rabc[x,s,y]] + 
          (2 (deltaxyz[s,x,y]^(1/2))/s) PolyLog[2,-x rabc[x,y,s]/deltaxyz[s,x,y]^(1/2)] + 
          (deltaxyz[s,x,y]^(1/2))/(s) 
          (
           ( Log[x tabc[x,y,s]/deltaxyz[s,x,y]^(1/2)] +2 Log[rabc[x,y,s]] )^2 + 
           (1 - ln[y,qq])Log[rabc[x,y,s]] +2 Log[y/x] Log[deltaxyz[s,x,y]^(1/2)/x] + 
           ( 5 ln[x,qq]ln[y,qq] -3 ln[x,qq]^2- 2 ln[y,qq]^2 + Log[x/y]) /2 + 
           2 Zeta[2] ) + 
          ((s-x-2 deltaxyz[s,x,y]^(1/2))/s) Log[rabc[x,y,s]]^2 + 
          (1-x/s) Log[x/y] Log[rabc[x,y,s]] + 
          (1-ln[x,qq]) ((x/s-y/s) Log[x/y] -(1-ln[y,qq]))/2 
          ),
(* Eq. (6.1) of [INTEGRALS] *)
     tabc[a_, b_, c_] -> (a+b-c + deltaxyz[a,b,c]^(1/2))/(2a),
     rabc[a_, b_, c_] -> (a+b-c - deltaxyz[a,b,c]^(1/2))/(2a),
(* Eq. (2.29) of [INTEGRALS] *)
     deltaxyz[x_, y_, z_] -> x^2 + y^2 + z^2 - 2 x y -2 x z -2 y z
    };

(* }}} *)



(* {{{ self-energies, evaluated on-shell *)

(* === one-loop gluino self-energy, Eq. (5.2) of [FERMION] === *)
piTildeGl1[mu_] := ca mgl^2 (10 - 6 ln[mgl^2,mu^2]) + 
    SumDep[ 4 tr ( BFS[quark[q], squark[q,j], mgl^2, mu^2] 
               - 2 L[q,j] R[q,j] Sqrt[quark[q]] mgl BFbarS[q, sq[q,j], mgl^2, mu^2]  ),
         {q, {U,D,S,C,T,B}},
         {j, {1,2}}];

(* === one-loop squark self-energy, Eq. (4.7) of [SCALAR] === *)
(* the factor of g_3^2 is moved to the coefficient, in order to
 * use a uniform notation for squarks and gluinos
 *)
piTildeSq1[q_, i_, j_, mu_] := cf SumDep[
    PP[q, i, k] PP[q, k, j] TsilA[squark[q,k], mu^2]
    + 2 delta[i, j] BFF[quark[q], mgl^2, squark[q, i], mu^2]
    - 2 NN[q, i, j] Sqrt[quark[q]] mgl BFbarFbar[quark[q], mgl^2, squark[q, i], mu^2]
    + delta[i, j] BSV[squark[q, i], 0, squark[q, i], mu^2]
    , {k, {1,2}}];
(* === two-loop squark self-energy, [SCALAR] Eqs. (4.7 - 4.10, 4.12) === *)
(* as above, the factor of g_3^4 will be absorbed in the coefficient *)
(* ---------------------------------------------------------------------- *)
piSq20[q_, i_, j_, s_, mu_] := SumDep[
    4 cf^2 (
            ((1)/(4)) PP[q, i, ki] PP[q, ki, mi] PP[q, mi, ni] PP[q, ni, j] *
            XSSS[squark[q, ki], squark[q, ni],squark[q, mi], mu^2]
            +  ((1)/(2)) PP[q, i, ki] PP[q, ki, j] *
            WSSFF[squark[q, ki],squark[q, ki],quark[q],mgl^2, mu^2]
(* \nonumber \\ &&                                                   *)
            -  ((1)/(2)) PP[q, i, ki] NN[q, ki, mi] PP[q, mi, j] *
            Sqrt[quark[q]] mgl *
            WSSFbarFbar[squark[q, ki],squark[q, mi],quark[q],mgl^2, mu^2]
            +  
            (L[q, i] L[q, j] L[q, ki]^2  
             +R[q, i] R[q, j] R[q, ki]^2) *
            VFFFFS[mgl^2,quark[q],quark[q],mgl^2,squark[q, ki],s,mu^2]
(* \nonumber \\ &&                                                   *)
            - delta[i,j] NN[q, ki, ki] *
            Sqrt[quark[q]] mgl *
            VFFFbarFbarS[mgl^2,quark[q],quark[q],mgl^2,squark[q, ki],s,mu^2]
            +  
            (L[q, i] R[q, j] R[q, ki] L[q, ki]
             +R[q, i] L[q, j] L[q, ki] R[q, ki])  mgl^2 *
            VFbarFFFbarS[mgl^2,quark[q],quark[q],mgl^2,squark[q, ki],s,mu^2]
(* \nonumber \\ &&                                                   *)
            + 
            (L[q, i] L[q, j] R[q, ki]^2
             +R[q, i] R[q, j] L[q, ki]^2) *
            quark[q] VFFbarFbarFS[mgl^2,quark[q],quark[q],mgl^2,squark[q, ki],s,mu^2]
            -NN[q, i, j] Sqrt[quark[q]] mgl *
            VFbarFFbarFS[mgl^2,quark[q],quark[q],mgl^2,squark[q, ki],s,mu^2]
(* \nonumber \\ &&                                                   *)
            +
            (L[q, i] R[q, j] L[q, ki] R[q, ki]
             +R[q, i] L[q, j] R[q, ki] L[q, ki]) quark[q] mgl^2 *
            VFbarFbarFbarFbarS[mgl^2,quark[q],quark[q],mgl^2,squark[q, ki],s,mu^2]
            )
(* \nonumber \\ &&                                                   *)
        +  (4 cf^2 - 2ca cf) * 
            (
             (L[q, i] R[q, j] R[q, ki] L[q, ki]
              +R[q, i] L[q, j] L[q, ki] R[q, ki]) *
             MFFFFS[quark[q],mgl^2,mgl^2,quark[q],squark[q, ki],s,mu^2]
(* \nonumber \\ &&                                                   *)
             - NN[q, i, j] Sqrt[quark[q]] mgl *
             MFFFbarFbarS[quark[q],mgl^2,mgl^2,quark[q],squark[q, ki],s,mu^2]
             + 
             (L[q, i] L[q, j] L[q, ki]^2  
              +R[q, i] R[q, j] R[q, ki]^2) mgl^2 *
             MFFbarFbarFS[quark[q],mgl^2,mgl^2,quark[q],squark[q, ki],s,mu^2]
(* \nonumber \\ &&                                                   *)
             - delta[i,j] NN[q, ki, ki] Sqrt[quark[q]] mgl *
             MFFbarFFbarS[quark[q],mgl^2,mgl^2,quark[q],squark[q, ki],s,mu^2]
             + 
             (L[q, i] L[q, j] R[q, ki]^2  
              +R[q, i] R[q, j] L[q, ki]^2) Sqrt[quark[q]]^2*
             MFFbarFbarFS[mgl^2,quark[q],quark[q],mgl^2,squark[q, ki],s,mu^2]
(* \nonumber \\ &&                                                   *)
             + 
             (L[q, i] R[q, j] L[q, ki] R[q, ki]  
              +R[q, i] L[q, j] R[q, ki] L[q, ki]) Sqrt[quark[q]]^2 mgl^2 *
             MFbarFbarFbarFbarS[quark[q],mgl^2,mgl^2,quark[q],squark[q, ki],s,mu^2]
             + ((1)/(4)) PP[q, i, ki] PP[q, ki, mi] PP[q, mi, ni] PP[q, ni, j] *
             SSSS[squark[q, ki], squark[q, mi],squark[q, ni],s,mu^2]
             )
        + 4 cf tr *
            (
             delta[i,j] *
             VFFFFS[quark[q],mgl^2,mgl^2,quark[r],squark[r, ri],s,mu^2]
(* %\nonumber \\ &&                                                  *)
             + delta[i,j] mgl^2 *
             VFFbarFbarFS[quark[q],mgl^2,mgl^2,quark[r],squark[r, ri],s,mu^2]
(* %\nonumber \\ &&                                                  *)
             - 2 NN[q, i, j] Sqrt[quark[r]] mgl *
             VFbarFFbarFS[quark[q],mgl^2,mgl^2,quark[r],squark[r, ri],s,mu^2]
(* \nonumber \\ &&                                                   *)
             + NN[q, i, j] NN[q, ri, ri] Sqrt[quark[q]] Sqrt[quark[r]] *
             VFbarFFFbarS[quark[q],mgl^2,mgl^2,quark[r],squark[r, ri],s,mu^2]
             - 2 delta[i,j] NN[q, ri, ri] Sqrt[quark[r]] mgl *
             VFFFbarFbarS[quark[q],mgl^2,mgl^2,quark[r],squark[r, ri],s,mu^2]
(* \nonumber \\ &&                                                   *)
             + NN[q, i, j] NN[q, ri, ri] Sqrt[quark[q]] Sqrt[quark[r]] mgl^2 *
             VFbarFbarFbarFbarS[quark[q],mgl^2,mgl^2,quark[r],squark[r, ri],s,mu^2]
             + ((1)/(4)) PP[q, i, ki] PP[q, ki, j] PP[q, ri, si] PP[q, si, ri] *
             SSSS[squark[q, ki], squark[r, ri],squark[ss, si],s,mu^2]
             ),
{r,{U,D,S,C,T,B}},
{ss,{U,D,S,C,T,B}},
{ri,{1,2}},
{si,{1,2}},
{ki,{1,2}},
{mi,{1,2}},
{ni,{1,2}}];

(* ---------------------------------------------------------------------- *)
piSq21[q_, i_, j_, s_, mu_] := SumDep[
(* \Pi^{(2,1)j}_{i} &=&                                              *)
    delta[i,j] *
        (
         cf ca ( GFF[quark[q], mgl^2,s,mu^2] + 
                GSSFF[squark[q, i],squark[q, i],quark[q], mgl^2,s,mu^2] ) + 
         (2 cf - ca) cf *
         GSSFF[squark[q, i],squark[q, i],mgl^2, quark[q],s,mu^2]  
         ) -
    NN[q, i, j] Sqrt[quark[q]] mgl *
        (
         cf ca ( GFbarFbar[quark[q], mgl^2,s,mu^2] + 
                GSSFbarFbar[squark[q, i],squark[q, j],quark[q], mgl^2,s,mu^2] ) + 
         (2 cf - ca) cf *
         GSSFbarFbar[squark[q, i],squark[q, j],mgl^2, quark[q],s,mu^2]
         ) + 
    cf^2 PP[q, i, ki] PP[q, ki, j] *
        (
         GS[squark[q, ki],mu^2] + GSSS[squark[q, i],squark[q, j],squark[q, ki],mu^2]
         ),
    {ki,{1,2}}
    ];
(* ---------------------------------------------------------------------- *)
piSq22[q_, i_, j_, s_, mu_] := SumDep[
(* \Pi^{(2,2)j}_i = *)
    delta[i,j] cf *
        (
         cf F1[squark[q, i],mu^2]
         + ca F2[squark[q, i],mu^2]
         + ca F3[squark[q, i], mgl^2,mu^2]
         + tr (F3[squark[q, i],quark[r],mu^2] + 
               F4[squark[q, i],squark[r, ri],mu^2] 
               )
         ),
    {r,{U,D,S,C,T,B}},
    {ri,{1,2}}
    ];
(* ---------------------------------------------------------------------- *)
piTildeSq20[q_, i_, mu_] := 
    (
     piSq20[q, i, i, squark[q,i], mu] +
     piTildeSq1[q, i, i, mu] 2 cf *
     (BFFp[quark[q], mgl^2, squark[q,i], mu^2]
      - NN[q, i, i] Sqrt[quark[q]] mgl BFbarFbarp[quark[q], mgl^2, squark[q,i], mu^2]
      )
     );
(* ---------------------------------------------------------------------- *)
piTildeSq21[q_, i_, mu_] := 
    (piSq21[q, i, i, squark[q,i], mu]/.
     {
          GSSS->GTildeSSS,
          GSSFF->GTildeSSFF,
          GSSFbarFbar->GTildeSSFbarFbar
      }
     );
piTildeSq22[q_, i_, mu_] :=
    (piSq22[q, i, i, squark[q,i], mu]/.
     {
          F1->FTilde1,
          F2->FTilde2,
          F3->FTilde3,
          F4->FTilde4
          }
     );

(* }}} *)

(* {{{ finally, define the conversion factors *)

convertOStoDRbar[Mgl_, Mst1_, Mst2_, Msq_, Mu_, Al4p_] := Module[
    {repm={mgl->Mgl,
           mst1->Mst1,
           mst2->Mst2,
           msq->Msq,
           mu->Mu
           },
     zgl1,
     zst11,
     zst21,
     repm1l,
     repm2l
     },

    print["Al4p part of conversion factors"];
    zgl1 = Re[trim[(piTildeGl1[mu]/mgl^2)/.su3/.toTSIL/.repm]/2];
    print[zgl1];
    zst11 = trim[(piTildeSq1[T, 1, 1, mu]/squark[T,1])/.su3/.toTSIL/.repm]/2;
    print[zst11];
    zst21 = trim[(piTildeSq1[T, 2, 2, mu]/squark[T,2])/.su3/.toTSIL/.repm]/2;
    print[zst21];

    repm1l = {mgl -> (1 - Al4p zgl1) Mgl,
              mst1 -> (1 - Al4p zst11) Mst1,
              mst2 -> (1 - Al4p zst21) Mst2,
              msq -> Msq,
              mu -> Mu
              };
    print[repm1l];
                  
    print["iterated Al4p part of conversion factors"];
    zgl1it = trim[(piTildeGl1[mu]/mgl^2)/.su3/.toTSIL/.repm1l]/2;
    print[zgl1it];
    zst11it = trim[(piTildeSq1[T, 1, 1, mu]/squark[T,1])/.su3/.toTSIL/.repm1l]/2;
    print[zst11it];
    zst21it = trim[(piTildeSq1[T, 2, 2, mu]/squark[T,2])/.su3/.toTSIL/.repm1l]/2;
    print[zst21it];

    zst12 = trim[
        ((
          piTildeSq20[T,1,mu] +
          piTildeSq21[T,1,mu] +
          piTildeSq22[T,1,mu] +
         (piTildeSq1[T,1,2,mu] piTildeSq1[T,2,1,mu])/(squark[T,1]-squark[T,2])
          )/squark[T,1])/.su3//.toTSIL/.repm]/2;
    print[zst12];
    zst22 = trim[
        ((
          piTildeSq20[T,2,mu] +
          piTildeSq21[T,2,mu] +
          piTildeSq22[T,2,mu] +
         (piTildeSq1[T,2,1,mu] piTildeSq1[T,1,2,mu])/(squark[T,2]-squark[T,1])
          )/squark[T,2])/.su3//.toTSIL/.repm]/2;
    print[zst22];

    repm2l = 
    {
         mst1 -> (1 
                  - Al4p zst11it
                  + Al4p^2 (zst11^2 - zst12) 
                  ) Mst1,
         mst2 -> (1 
                  - Al4p zst21it
                  + Al4p^2 (zst21^2 - zst22) 
                  ) Mst2
                  };
    Return[repm2l];
];

(* toOSgl[1] := mgl -> Mgl ( 1 - Al4p zgl1 );                        *)
(* toOSst1[1] := mst1 -> Mst1 ( 1 - Al4p zst11 );                    *)
(* toOSst2[1] := mst2 -> Mst2 ( 1 - Al4p zst21 );                    *)

(* toOS[1] := {toOSgl[1],                                            *)
(*             toOSst1[1],                                           *)
(*             toOSst2[1]                                            *)
(*         };                                                        *)

(* zst12 :=                                                          *)
(*     (                                                             *)
(*      piTildeSq20[T,1,mu] +                                        *)
(*      piTildeSq21[T,1,mu] +                                        *)
(*      piTildeSq22[T,1,mu] +                                        *)
(*      (piTildeSq1[T,1,2,mu] piTildeSq1[T,2,1,mu])/(Squark[T,1]-Squark[T,2]) *)
(*      );                                                           *)
(* zst22 :=                                                          *)
(*     (                                                             *)
(*      piTildeSq20[T,2,mu] +                                        *)
(*      piTildeSq21[T,2,mu] +                                        *)
(*      piTildeSq22[T,2,mu] +                                        *)
(*      (piTildeSq1[T,2,1,mu] piTildeSq1[T,1,2,mu])/(Squark[T,2]-Squark[T,1]) *)
(*      );                                                           *)



(* }}} *)





(* how to evaluate piSq20 to a finite value:

In[1]:= <</users/ttp/otter/q2eexpcalc3/calc3/problems/c_mh/math/tsil/polemasses.m
In[5]:= Simplify[Expand[piSq20[T,1,1,s,mu]/.toTSIL/.mgl->Mst],Mst!=0]
In[6]:= %/.{Mst->1000.,Msq->2000.,s->1000.^2,mu->1000.}

 *)





