#include <stdio.h>
#include "internal.h"
#include "self_fermion.h"

long double strtold( const char *, char** );

#define PRECISION_GOAL 0.05L

/* Facts about QCD: */
#define CG 3.L
#define Cq (4.L/3.L)
#define Iq 0.5L

/* Other definitions */
/* #define PI 3.1415926535897932385L                                 */
/* #define PI2 9.8696044010893586188L                                */
/* #define ln2 0.69314718055994530942L                               */
#define kappa (1.L/(16.L*PI2))
/* #define Zeta3 1.2020569031595942854L                              */

struct MSSM_data {
  TSIL_REAL qq; /* Renormalization scale squared */
  TSIL_REAL g3; /* QCD gauge coupling */
  TSIL_REAL gluino; /* Gluino running squared mass */
  TSIL_REAL quark[6]; /* Quark squared masses in the order d,u,s,c,b,t */
  TSIL_REAL squark[6][2]; /* Squark squared masses */
  TSIL_COMPLEX Rsq[6][2]; /* Squark mixing matrices */
  TSIL_COMPLEX Lsq[6][2]; /* Squark mixing matrices */
  TSIL_REAL MtsqOS; // squared top mass in the on-shell scheme
};

typedef struct MSSM_data MSSM_DATA;

int quarkpole (MSSM_DATA *mssm_data,
               TSIL_REAL *Mpole1,
               TSIL_REAL *Mpole2);

/* ------------------------------------------------------------------- */

int main ( int argc, char *argv[] )
{
  MSSM_DATA mssm_data;
  int i,j;
  TSIL_REAL Mpole1, Mpole2;
  TSIL_REAL delta;
/*   TSIL_REAL rescalefactor;                                        */

  fclose(stderr);  /* Suppress irrelevant warning messages. */

  if( argc != 8 )
    TSIL_Error( "quarkpole",
                "Need 7 arguments: alpha_s, MtOS, Mgl, Mst1, Mst2, Msq, stheta", 1);

  // strong coupling
  mssm_data.g3 = 
    TSIL_SQRT(((TSIL_REAL) strtold( argv[1], (char **) NULL ))*4.L*PI);
  // top mass squared
  mssm_data.MtsqOS = 
    TSIL_POW((TSIL_REAL) strtold( argv[2], (char **) NULL ), 2);
  // gluino mass squared
  mssm_data.gluino = 
    TSIL_POW(((TSIL_REAL) strtold( argv[3], (char **) NULL )), 2);


  // massless light quarks
  mssm_data.quark[0] = 0.L; 
  mssm_data.quark[1] = 0.L; 
  mssm_data.quark[2] = 0.L;
  mssm_data.quark[3] = 0.L;
  mssm_data.quark[4] = 0.L;

  /* top squark masses and mixings */
  mssm_data.squark[5][0] = 
    TSIL_POW(((TSIL_REAL) strtold( argv[4], (char **) NULL )), 2); 
  mssm_data.squark[5][1] = 
    TSIL_POW(((TSIL_REAL) strtold( argv[5], (char **) NULL )), 2); 
  mssm_data.Lsq[5][0] = ((TSIL_REAL) strtold( argv[7], (char **) NULL ));
  mssm_data.Rsq[5][1] = - mssm_data.Lsq[5][0];
  mssm_data.Rsq[5][0] = TSIL_SQRT(1 - TSIL_POW(mssm_data.Lsq[5][0], 2));
  mssm_data.Lsq[5][1] = TSIL_SQRT(1 - TSIL_POW(mssm_data.Lsq[5][0], 2));

  /* u,d,s,c,b squark masses and mixings */
  for (i=0;i<5;i++) { 
    mssm_data.squark[i][0] = 
      TSIL_POW(((TSIL_REAL) strtold( argv[6], (char **) NULL )), 2); 
    mssm_data.squark[i][1] = 
      TSIL_POW(((TSIL_REAL) strtold( argv[6], (char **) NULL )), 2); 
    mssm_data.Rsq[i][0] = 1.L; 
    mssm_data.Lsq[i][0] = 0.L;
    mssm_data.Rsq[i][1] = 0.L; 
    mssm_data.Lsq[i][1] = 1.L;
  }


  // define mssm_data.quark[5] and mssm_data.qq iteratively in the loop

  mssm_data.quark[5] = mssm_data.MtsqOS; 
  mssm_data.qq = mssm_data.quark[5];

  quarkpole( &mssm_data, &Mpole1, &Mpole2 );

  printf("alpha_s               :     %.16lf\n",
         (double) (TSIL_POW(mssm_data.g3, 2)/(4.0L*PI))
         );
  printf("g_3                   :     %.16lf\n",
         (double) mssm_data.g3
         );
  printf("Top mass on-shell           %.16lf\n", 
         (double) TSIL_SQRT(mssm_data.quark[5])); 

  printf("Tree-level gluino mass:     %.16lf\n", 
         (double) TSIL_SQRT(mssm_data.gluino)); 
  for (i=0; i<6; i++){
   for (j=0; j<2; j++){
    if ((i==0) && (j==0))
      printf("Tree-level squark masses:   %.16lf\n", 
          (double) TSIL_SQRT(mssm_data.squark[i][j])); 
    else 
      printf("                            %.16lf\n", 
          (double) TSIL_SQRT(mssm_data.squark[i][j])); 
  }}
  printf("One-loop top pole mass:  %.16lf\n", (double) Mpole1);
  printf("Two-loop top pole mass:  %.16lf\n", (double) Mpole2);
  printf("Delta    top pole mass:  %.16lf\n", (double) (Mpole2 - Mpole1));


  do {

    delta = 
      TSIL_FABS( TSIL_SQRT(mssm_data.quark[5])
                 - TSIL_SQRT(TSIL_POW(Mpole2, 2)) );
    
    printf("delta: %.16lf\n", (double) delta);
    mssm_data.quark[5] = TSIL_POW(Mpole2, 2);
    mssm_data.qq = mssm_data.quark[5];
    
    quarkpole( &mssm_data, &Mpole1, &Mpole2 );
    printf("One-loop top pole mass:  %.16lf\n", (double) Mpole1);
    printf("Two-loop top pole mass:  %.16lf\n", (double) Mpole2);
    printf("Delta    top pole mass:  %.16lf\n", (double) (Mpole2 - Mpole1));

  } while ( delta > PRECISION_GOAL );

  return 0;



}

int quarkpole (MSSM_DATA *mssm_data,
               TSIL_REAL *Mpole1, 
               TSIL_REAL *Mpole2)
{
  TSIL_REAL    x, y, z, g3, qq, gluino, quark[6], squark[6][2];
  TSIL_REAL    mgluino, mquark[6];
  TSIL_COMPLEX Rsq[6][2], Lsq[6][2], Rsqc[6][2], Lsqc[6][2], reLRc[6][2];
  TSIL_DATA    int_data[6][2][2];
  TSIL_DATA    int_data2[6][2][2];
  TSIL_COMPLEX deltas1, s1, deltas2, s2;
  TSIL_COMPLEX pi1Vtilde, pi1Stilde, pi1leftover;
  TSIL_COMPLEX pi2tildeMSFFSF, pi2tildeVSFFFS, pi2tildeVFSSFF;
  TSIL_COMPLEX pi2tildeYFSSS, pi2tildeBFS, pi2tildef, pi2tildeF;
  TSIL_COMPLEX BFS[5][2], BfS[5][2], BpFS, BpfS;
  TSIL_COMPLEX Lj, Ljc, Rj, Rjc, Lk, Lkc, Rk, Rkc, Ln, Lnc, Rn, Rnc;
  TSIL_COMPLEX MSFFSF, MSFFSf, MSFfSF, MSFfSf, MSffSF, MSffSf;
  TSIL_COMPLEX VSFFFS, VSFFfS, VSFfFS, VSFffS, VSffFS, VSfffS;
  TSIL_COMPLEX VFSSFF, VFSSff, VfSSFF, VfSSff;
  TSIL_COMPLEX YFSSS, YfSSS;
  TSIL_COMPLEX f1result, f2result, f3result, f4result, f5result, f6result;
  TSIL_COMPLEX BFV, BfV;
  int i,j,k,n;

  /* Read in the parameters from the mssm_data struct. */
  g3 = mssm_data->g3;
  qq = mssm_data->qq;
  gluino = mssm_data->gluino;
  mgluino = TSIL_SQRT(gluino);

  for (i=0;i<6;i++) { // i denotes the quark flavours
    quark[i] = mssm_data->quark[i];
    mquark[i] = TSIL_SQRT (quark[i]);
    for (j=0;j<2;j++) {
      squark[i][j] = mssm_data->squark[i][j];
/*          printf("squark[%i][%i]:   %.16lf\n", i, j,                 */
/*            (double) (squark[i][j]));                                */
      Rsq[i][j] = mssm_data->Rsq[i][j]; // R_sq
      Lsq[i][j] = mssm_data->Lsq[i][j]; // L_sq
      Rsqc[i][j] = TSIL_CREAL(Rsq[i][j]) - I*TSIL_CIMAG(Rsq[i][j]); // R_sq^*
      Lsqc[i][j] = TSIL_CREAL(Lsq[i][j]) - I*TSIL_CIMAG(Lsq[i][j]); // L_sq^*
      reLRc[i][j] = TSIL_CREAL(Lsq[i][j]*Rsqc[i][j]); // Re[L_sq R_sq^*]
    }}
  x = quark[5];
  /* One-loop part starts here. */

  pi1Vtilde = Cq * x * ( 10.0L - 6.0L * TSIL_LOG(x/qq) );




  pi1Stilde = 0.L;

  for (j=0; j<2; j++){
      bFS (gluino,squark[5][j],x,qq,
           &(BFS[5][j]), &(BfS[5][j]));

    pi1Stilde += Cq * 
      ( 2.0L * BFS[5][j]
        - 4.0L * reLRc[5][j] * mquark[5] * mgluino * BfS[5][j]
        );
  }

  /* One-loop part ends here. */

  /* Two-loop part starts here. */

  /* part containing MSFFSF */
  pi2tildeMSFFSF = 0.L;
     
  for (j=0; j<2; j++){
    for (k=0; k<2; k++){
      TSIL_SetParameters 
        (&(int_data[5][j][k]), 
         squark[5][j], 
         gluino, 
         gluino, 
         squark[5][k], 
         x, 
         qq);
      TSIL_Evaluate (&int_data[5][j][k], x);

      mSFFSF( &int_data[5][j][k], 0,
              &MSFFSF, &MSFFSf, 
              &MSFfSF, &MSFfSf, 
              &MSffSF, &MSffSf);

      pi2tildeMSFFSF += Cq*( 8*Cq - 4*CG ) * 
        (
         Lsq[5][j]*Rsqc[5][j]*Lsqc[5][k]*Rsq[5][k] * MSFFSF
         + TSIL_POW(TSIL_CABS(Lsq[5][j]*Rsq[5][k]), 2)* x * MSFFSf
         - reLRc[5][k] * mquark[5]*mgluino * MSFfSF
         + 0.5L* (TSIL_POW(TSIL_CABS(Lsq[5][j]*Lsq[5][k]), 2)  
                  + TSIL_POW(TSIL_CABS(Rsq[5][j]*Rsq[5][k]), 2)
                  )* gluino * MSffSF
         - reLRc[5][j] * mquark[5]*mgluino * MSFfSf
         + TSIL_CREAL( Lsq[5][j]*Rsqc[5][j]*Lsq[5][k]*Rsqc[5][k] 
                       ) * x * gluino * MSffSf
          
         );
    }
  }
    printf("pi2tildeMSFFSF: %.16lf\n", (double) pi2tildeMSFFSF);
  
  /* part containing VSFFFS */

/*   printf("squark[0][1]:   %.16lf\n", (double) (squark[0][1]));                                */
/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
  for (i=0;i<6;i++) { // i denotes the quark flavours
    quark[i] = mssm_data->quark[i];
    mquark[i] = TSIL_SQRT (quark[i]);
    for (j=0;j<2;j++) {
      squark[i][j] = mssm_data->squark[i][j];
      Rsq[i][j] = mssm_data->Rsq[i][j]; // R_sq
      Lsq[i][j] = mssm_data->Lsq[i][j]; // L_sq
      Rsqc[i][j] = TSIL_CREAL(Rsq[i][j]) - I*TSIL_CIMAG(Rsq[i][j]); // R_sq^*
      Lsqc[i][j] = TSIL_CREAL(Lsq[i][j]) - I*TSIL_CIMAG(Lsq[i][j]); // L_sq^*
      reLRc[i][j] = TSIL_CREAL(Lsq[i][j]*Rsqc[i][j]); // Re[L_sq R_sq^*]
    }}
/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */

  pi2tildeVSFFFS = 0.L;


  for (i=0; i<6; i++){
    for (j=0; j<2; j++){
      for (k=0; k<2; k++){
        TSIL_SetParameters
          (&(int_data[i][j][k]),
           squark[5][j],
           1000.0L,
           gluino,
           quark[i],
           squark[i][k],
           qq);

        TSIL_Evaluate (&int_data[i][j][k], x);
      
        vSFFFS (&int_data[i][j][k], &int_data[i][j][k], 0, 
              &VSFFFS, &VSFFfS, &VSFfFS, &VSFffS, &VSffFS, &VSfffS);
        
        pi2tildeVSFFFS += 8.0L*Cq*Iq*
          (
           0.5L*VSFFFS
           + 2.0L*TSIL_CREAL( 
                             Lsq[5][j]*Rsqc[5][j]* 
                             Lsqc[i][k]*Rsq[i][k]
                             )*mquark[5]*mquark[i]*VSFFfS
           - 2.0L*reLRc[5][j]*mquark[5]*mgluino*VSFfFS
           + 0.5L*gluino*VSffFS
           - 2.0L*reLRc[i][k]*mquark[i]*mgluino* VSFffS  
           + 2.0L*TSIL_CREAL(Lsq[5][j]*Rsqc[5][j]*
                             Lsq[i][k]*Rsqc[i][k])*
           mquark[i]*mquark[5]*gluino*VSfffS  
           );           

      }}}

    printf("pi2tildeVSFFFS: %.16lf\n", (double) pi2tildeVSFFFS);

  /* part containing VFSSFF */
/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
/*    for (i=0;i<6;i++) { // i denotes the quark flavours              */
/*     quark[i] = mssm_data->quark[i];                               */
/*     mquark[i] = TSIL_SQRT (quark[i]);                             */
/*      for (j=0;j<2;j++) {                                            */
/*       squark[i][j] = mssm_data->squark[i][j];                     */
/*       Rsq[i][j] = mssm_data->Rsq[i][j]; // R_sq                   */
/*       Lsq[i][j] = mssm_data->Lsq[i][j]; // L_sq                   */
/*       Rsqc[i][j] = TSIL_CREAL(Rsq[i][j]) - I*TSIL_CIMAG(Rsq[i][j]); // R_sq^* */
/*       Lsqc[i][j] = TSIL_CREAL(Lsq[i][j]) - I*TSIL_CIMAG(Lsq[i][j]); // L_sq^* */
/*       reLRc[i][j] = TSIL_CREAL(Lsq[i][j]*Rsqc[i][j]); // Re[L_sq R_sq^*] */
/*        printf("squark[][]:   %.16lf\n", (double) (squark[i][j]));                                  */
/*      }}                                                             */
/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */

  pi2tildeVFSSFF = 0.L;

    for (j=0; j<2; j++){
      for (k=0; k<2; k++){
        TSIL_SetParameters
          (&(int_data[5][j][k]),
           gluino,
           squark[5][j],
           squark[5][k],
           1000.0L,
           gluino,
           qq);
        TSIL_SetParameters
          (&(int_data2[5][j][k]),
           quark[5],
           squark[5][j],
           squark[5][k],
           1000.0L,
           gluino,
           qq);

        TSIL_Evaluate (&int_data[5][j][k], x);
        TSIL_Evaluate (&int_data2[5][j][k], x);
      
        vFSSFF (&int_data[5][j][k], &int_data2[5][j][k],
                &VFSSFF, &VFSSff, 
                &VfSSFF, &VfSSff);


        if( j == k ){
          pi2tildeVFSSFF += 4.0L*Cq*Cq*
            (
             VFSSFF 

             - 2.0L*reLRc[5][j]*mquark[5]*mgluino*(
                                                   VfSSFF 
                                                   +VFSSff 
                                                   )
             );
        }

        pi2tildeVFSSFF += 4.0L*Cq*Cq*
          (
           + 2*(TSIL_CREAL(Lsq[5][j]*Rsqc[5][j]*Lsq[5][k]*Rsqc[5][k])
                + TSIL_POW(TSIL_CABS(Lsq[5][j]*Rsq[5][k]), 2))*
           quark[5]*gluino*VfSSff 
           );

  }}

    printf("pi2tildeVFSSFF: %.16lf\n", (double) pi2tildeVFSSFF);

    /* part containing pi2tildeYFSSS */

    pi2tildeYFSSS = 0.L;

    for (j=0; j<2; j++){
      for (k=0; k<2; k++){

        yFSSS (gluino, squark[5][j], squark[5][j], squark[5][k],
               x, qq,
               &YFSSS, &YfSSS);
        
        pi2tildeYFSSS += 4.0L*Cq*Cq*
          (
           0.5L* TSIL_POW(TSIL_CABS(Lsq[5][j]*Lsqc[5][k]
                                    -Rsq[5][j]*Rsqc[5][k]), 2)*YFSSS

           );

        for( n=0; n<2; n++ ){
        yFSSS (gluino, squark[5][j], squark[5][k], squark[5][n],
               x, qq,
               &YFSSS, &YfSSS);
          pi2tildeYFSSS += 4.0L*Cq*Cq*
          (
           -TSIL_CREAL
           (Lsqc[5][j]*Rsq[5][k]*
            (Lsq[5][j]*Lsqc[5][n]-Rsq[5][j]*Rsqc[5][n])*
            (Lsq[5][n]*Lsqc[5][k]-Rsq[5][n]*Rsqc[5][k]))*
           mquark[5]*mgluino*YfSSS
           );
        }

      }
    }

    printf("pi2tildeYFSSS: %.16lf\n", (double) pi2tildeYFSSS);
    
    /* part containing BFS */

    pi2tildeBFS = 0.L;
        for (j=0; j<2; j++){
      bFS (gluino, squark[5][j], x, qq,
           &BFS[5][j], &BfS[5][j]);
    }
    for (j=0; j<2; j++){
      for (k=0; k<2; k++){

        bpFS (gluino, squark[5][k], x, qq,
              &BpFS, &BpfS);


        pi2tildeBFS += 4.0L*Cq*Cq*
          (
           ( BFS[5][j]
             - 2.0L*reLRc[5][j]*mquark[5]*mgluino*BfS[5][j]
             ) *
           ( BpFS
             - 2.0L*reLRc[5][k]*mquark[5]*mgluino*BpfS
             )
           + reLRc[5][j]*reLRc[5][k]*gluino*BfS[5][j]*BfS[5][k]
           -TSIL_POW(TSIL_CABS(Lsq[5][j]*Rsq[5][k]), 2)*BFS[5][j]*BFS[5][k]/(1.0L*quark[5])
           );

      }
      }
    printf("pi2tildeBFS: %.16lf\n", (double) pi2tildeBFS);

    /* part containing f */

    pi2tildef = 0.L;
    
    for (j=0; j<2; j++){

      f1f4 (quark[5], gluino, squark[5][j], qq, 
            &f1result, &f4result);
      f2f3f5f6 (quark[5], gluino, squark[5][j], qq, 
              &f2result, &f3result,
                &f5result, &f6result);

      pi2tildef += 
        (
         CG*Cq*f1result
         + 2*(CG - Cq)*Cq*f2result
         + 2*Cq*Cq*f3result
         - 2*Cq*reLRc[5][j]*mquark[5]*mgluino*
         (
          CG*f4result
          + 2*(CG -Cq)*f5result
          + 2*Cq*f6result
          )
         );
    }
    printf("pi2tildef: %.16lf\n", (double) pi2tildef);

    /* part containing F */

    pi2tildeF = Cq*Cq*F1(quark[5], qq)
      + CG*Cq*(F2(quark[5], qq) 
               + F3(quark[5], gluino, qq));

    printf("pi2tildeF: %.16lf\n", (double) pi2tildeF);

/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
  for (i=0;i<6;i++) { // i denotes the quark flavours
    quark[i] = mssm_data->quark[i];
    mquark[i] = TSIL_SQRT (quark[i]);
    for (j=0;j<2;j++) {
      squark[i][j] = mssm_data->squark[i][j];
      Rsq[i][j] = mssm_data->Rsq[i][j]; // R_sq
      Lsq[i][j] = mssm_data->Lsq[i][j]; // L_sq
      Rsqc[i][j] = TSIL_CREAL(Rsq[i][j]) - I*TSIL_CIMAG(Rsq[i][j]); // R_sq^*
      Lsqc[i][j] = TSIL_CREAL(Lsq[i][j]) - I*TSIL_CIMAG(Lsq[i][j]); // L_sq^*
      reLRc[i][j] = TSIL_CREAL(Lsq[i][j]*Rsqc[i][j]); // Re[L_sq R_sq^*]
    }}
/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
    printf("squark[0][0]:   %.16lf\n", (double) (squark[0][0]));
  for (i=0; i<6; i++){
    printf("F3(quark[5], quark[%i]: %.16lf\n", i, (double) F3(quark[5], quark[i], qq));
    for (j=0; j<2; j++){
    pi2tildeF += Cq*Iq*(2.0L*F3(quark[5], quark[i], qq) 
                        + F4(quark[5], squark[i][j], qq) );
    printf("F4(quark[5], squark[%i][%i]: %.16lf\n", i, j,(double) F4(quark[5], squark[i][j], qq));

  }
  }      

    printf("pi2tildeF: %.16lf\n", (double) pi2tildeF);
    
  /* Two-loop part ends here. */

  /* Assemble results to obtain the gluino pole mass */

  deltas1 = (1/x)*kappa*g3*g3*(pi1Vtilde + pi1Stilde);

  deltas2 = (1/x)*kappa*kappa*g3*g3*g3*g3*
    (
     pi2tildeMSFFSF
     + pi2tildeVSFFFS
     + pi2tildeVFSSFF
     + pi2tildeYFSSS
     + pi2tildeBFS
     + pi2tildef
     + pi2tildeF
     );

/*   s1 = mssm_data->MtsqOS - x*deltas1;                             */
/*   s2 = s1 + x*(                                                   */
/*                + deltas1*deltas1                                  */
/*                - deltas2                                          */
/*                );                                                 */

  s1 = mssm_data->MtsqOS*(1-deltas1);
  s2 = mssm_data->MtsqOS*(1-deltas1
                          -deltas2
                          );


  *Mpole1 = TSIL_SQRT(TSIL_CREAL(s1));

  *Mpole2 = TSIL_SQRT(TSIL_CREAL(s2));

  return 0;
}
